@@ -5,7 +5,7 @@ import operator
 import re
 import sys
 import typing
-from collections import deque
+from collections import Counter, deque
 
 from typing_extensions import ParamSpec
 
@@ -221,6 +221,65 @@ class list(builtins.list, typing.Generic[T]):
         """Return the length of this list."""
         return len(self)
 
+    def mean(self) -> T:
+        """Statistical mean of this list.
+
+        T must be summable and divisible by an integer,
+        and there must be at least one element in this list.
+        """
+        if self.len() == 0:
+            raise ValueError("Called mean() on an empty list")
+        return self.sum() / self.len()
+
+    def median(self) -> "T":
+        """Statistical median of this list.
+
+        T must be orderable and there must be at least one
+        element in this list.
+        Further more, if this list contains an odd number
+        of elements, T must also be summable and divisible
+        by an integer.
+        """
+        if self.len() == 0:
+            raise ValueError("Called median() on an empty list")
+        if self.len() % 2:
+            return self.sorted()[self.len() // 2]
+        else:
+            sorted_self = self.sorted()
+            return (sorted_self[self.len() // 2] + sorted_self[self.len() // 2 - 1]) / 2
+
+    def mode(self) -> "list[T]":
+        """Statistical mode of this list.
+
+        T must be hashable and there must be at least one
+        element in this list.
+        """
+        if self.len() == 0:
+            raise ValueError("Called mode() on an empty list")
+        counted = Counter(self).most_common()
+        n_ties = max(i[1] for i in counted)
+        return list(i[0] for i in counted if i[1] == n_ties)
+
+    def flat(self, recursive: bool = False) -> "list":
+        """Flattened version of this list.
+
+        If recursive is specified, flattens recursively instead
+        of by one layer.
+        """
+        if not recursive:
+            return list(item for list in self for item in list)
+        return list(
+            subitem
+            for item in self
+            for subitem in (
+                item.flatten(True)
+                if isinstance(item, iter)
+                else list(item).flat(True)
+                if isinstance(item, builtins.list)
+                else [item]
+            )
+        )
+
 
 class iter(typing.Generic[T]):
     """Smart/fluent iterator class"""
@@ -384,9 +443,19 @@ class iter(typing.Generic[T]):
         """
         return tuple(self.next() for _ in builtins.range(n))
 
-    def collect(self) -> "typing.List[T]":
+    @typing.overload
+    def collect(self) -> list[T]:
+        ...
+
+    @typing.overload
+    def collect(self, collection_type: typing.Type[U]) -> "U[T]":
+        ...
+
+    def collect(self, collection_type=None):
         """Return a list containing all remaining elements of this iterator."""
-        return list(self)
+        if collection_type is None:
+            collection_type = list
+        return collection_type(self)
 
     def chain(self, other: typing.Iterable[T]) -> "iter[T]":
         """Return an iterator containing the elements of this iterator followed
@@ -468,6 +537,26 @@ class iter(typing.Generic[T]):
         """
         return iter(itertools.combinations_with_replacement(self, r))
 
+    def flatten(self, recursive: bool = False) -> "iter":
+        """Flatten this iterator.
+
+        If recursive is specified, flattens recursively instead
+        of by one layer.
+        """
+        if not recursive:
+            return iter(item for iterator in self for item in iterator)
+        return iter(
+            item
+            for iterator in self
+            for item in (
+                iterator.flatten(True)
+                if isinstance(iterator, iter)
+                else list(iterator).flat(True)
+                if isinstance(iterator, builtins.list)
+                else iterator
+            )
+        )
+
     def __repr__(self) -> str:
         return f"Smart({self.it!r})"
 
@@ -477,6 +566,11 @@ def range(*args, **kw):
     return iter(builtins.range(*args, **kw))
 
 
+@functools.wraps(builtins.map)
+def map(*args, **kw):
+    return iter(builtins.map(*args, **kw))
+
+
 def irange(start: int, stop: int) -> iter[int]:
     """Inclusive range. Returns an iterator that
     yields values from start to stop, including both
@@ -486,7 +580,7 @@ def irange(start: int, stop: int) -> iter[int]:
     if start <= stop:
         return range(start, stop + 1)
     else:
-        return range(stop, start - 1, -1)
+        return range(start, stop - 1, -1)
 
 
 def _frange(
@@ -532,7 +626,7 @@ def tail_call(func: typing.Callable[P, U]) -> typing.Callable[P, U]:
     """
 
     @functools.wraps(func)
-    def g(*args, **kwargs):
+    def wrapped(*args, **kwargs):
         f = sys._getframe()
         if f.f_back and f.f_back.f_back and f.f_back.f_back.f_code == f.f_code:
             raise TailRecursionDetected(args, kwargs)
@@ -544,4 +638,4 @@ def tail_call(func: typing.Callable[P, U]) -> typing.Callable[P, U]:
                     args = e.args
                     kwargs = e.kwargs
 
-    return func
+    return wrapped
